function [hmm] = arhmm_load(fname)

file = fopen(fname);

% Integer parameters

K = fscanf(file, '%d', 1);
S = fscanf(file, '%d', 1);
D = fscanf(file, '%d', 1);

hmm.K = K;
hmm.S = S;
hmm.D = D;

% Prior, transition matrix

p = fscanf(file, '%f', S);
A = fscanf(file, '%f', [S S]);

hmm.p = p';
hmm.A = A';

% HMM Accumulators

p1_acc = fscanf(file, '%f', S);
A_acc  = fscanf(file, '%f', [S S]);

hmm.p1_acc = exp(p1_acc');
hmm.A_acc  = exp(A_acc');

% AR Processes

pt_acc = zeros(1,S);

for s=1:S
  % Parameters
  
  R = fscanf(file, '%d', 1);
  D = fscanf(file, '%d', 1);

  hmm.R = R;
    
  C{s} = zeros(D,D,R);
  
  % AR Coefficients
  
  for r=1:R
    tmp         = fscanf(file, '%f', [D D]);
    C{s}(:,:,r) = tmp';
  end
  
  % Covariance matrix
  
  tmp    = fscanf(file, '%f', [D D]);
  Cov{s} = tmp';

  % Accumulators
  
  pt_acc(s) = fscanf(file, '%f', 1);

  len = R*D;
  
  tmp        = fscanf(file, '%f', [len len]);
  B_acc{s}   = tmp';
  
  tmp        = fscanf(file, '%f', [len D]);
  d_acc{s}   = tmp';
  
  tmp        = fscanf(file, '%f', [D D]);
  Cov_acc{s} = tmp';
end

hmm.pt_acc = pt_acc;

hmm.C   = zeros(D,D,R,S);
hmm.Cov = zeros(D,D,S); 

hmm.B_acc   = zeros(len, len, S);
hmm.d_acc   = zeros(len, D, S);
hmm.Cov_acc = zeros(D, D, S);

for s=1:S
  for r=1:R
    hmm.C(:,:,r,s) = C{s}(:,:,r);
  end
  
  hmm.Cov(:,:,s) = Cov{s};
  
  hmm.B_acc(:,:,s)   = B_acc{s};
  hmm.d_acc(:,:,s)   = d_acc{s};
  hmm.Cov_acc(:,:,s) = Cov_acc{s};
end

fclose(file);
