function [] = demo_arslds()

addpath mex;
  
color = [    
    0.1991    0.2987    0.6614
    0.2844    0.4692    0.0648
    0.9883    0.5828    0.4235
    0.5155    0.3340    0.4329
    0.2259    0.5798    0.7604
    0.5298    0.6405    0.2091
    0.3798    0.7833    0.6808
    0.4611    0.5678    0.7942
    0.0592    0.6029    0.0503
    0.4154    0.3050    0.8744
    0.0150    0.7680    0.9708
    0.9901    0.7889    0.4387
    0.4983    0.2140    0.6435
    0.3200    0.9601    0.7266
    0.4120    0.7446    0.2679
    0.4399    0.9334    0.6833
    0.2126    0.8392    0.6288
    0.1338    0.2071    0.6072
    0.6299    0.3705    0.5751
    0.4514    0.0439    0.0272];

fs   = 8000;      % sampling frequency of the signal
covv = 1;         % large observation variance

% load the model

model = 'arhmm-1.dat';

disp(['Model    : ', model]);

hmm    = arhmm_load(model);
arslds = arslds_from_arhmm(hmm, covv);

S = arslds.S;
H = arslds.H;
K = arslds.K;

% load the utterance

utt = 'one.dat';

disp(['Utterance: ', utt]);

v = load(utt)';

% ...

T = size(v,2);
N = ceil(T / K);

range = [0:T-1] / fs;

% plot it

clf;
subplot(5,1,1);
plot(range, v, 'color', [0 0 0]);

title('Original signal');

pause(0.1);

% initial gains

gainH = zeros(S,N);
gainV = zeros(S,N);

for n=1:N
  for s=1:S
    gainH(s,n) = arslds.CovH(s);
    gainV(s,n) = arslds.CovV(s);
  end
end

for iter=1:50
  % go forward
  
  [logr, f, F, llh] = arslds_fwd(arslds, v, gainH, gainV);

  disp(['Log likelihood: ', num2str(llh)]);

  maxfwd = zeros(T,1);
  
  for t=1:T
    [val, ind] = max(logr(:,t));
    maxfwd(t)  = ind;
  end
  
  % plot the infered switches
  
  subplot(5,1,2);
  plot(range, v, 'color', 0.9*ones(3,1));
  
  hold on;
  for s=1:S
    ts = find(maxfwd == s);
    plot(range(ts), v(ts), '.', 'color', color(s,:));
  end
  hold off;
  
  title('Forward');

  pause(0.1);
  
  % force the last state to be S
  
  logr(:,T) = -inf*ones(S,1);
  logr(S,T) = 0.0;
  
  % go backward

  [logl, pair_mean, pair_2mom] = ...
      arslds_bwd(arslds, gainH, logr, f, F);
    
  clear logfpmix;
  clear F;
  
  maxbwd = zeros(T,1);
  maxfwd = zeros(T,1);
  
  for t=1:T
    [val, ind] = max(logr(:,t));
    maxfwd(t)  = ind;
    
    [val, ind] = max(logl(:,t));
    maxbwd(t)  = ind;
  end
  
  % plot the infered switches
  
  subplot(5,1,3);
  plot(range, v, 'color', 0.9*ones(3,1));
  
  hold on;
  for s=1:S
    ts = find(maxbwd == s);
    plot(range(ts), v(ts), '.', 'color', color(s,:));
  end
  hold off;

  title('Backward');
  
  pause(0.1);

  % plot the filtered signal
  
  subplot(5,1,4);
  v_filter = zeros(1,T);
  
  for t=1:T
    st = maxfwd(t);
    v_filter(:,t) = f(1,st,t);
  end
  
  plot(range, v_filter);
  
  title('Filtered signal');
  
  pause(0.1);

  % plot the smoothed signal
    
  subplot(5,1,5);
  v_smooth = zeros(1,T);
  
  for t=1:T
    st = maxbwd(t);
    v_smooth(:,t) = pair_mean(H+1,st,t);
  end
  
  plot(range, v_smooth);
  
  title('Smoothed signal');
  
  pause(0.1);
  
  % save the filtered and smoothed signal

  wavwrite(v, fs, 'original.wav');
  wavwrite(v_filter, fs, 'filtered.wav');
  wavwrite(v_smooth, fs, 'smoothed.wav');

  % adapt gainH and gainV
  
  gainH = update_gainH(arslds, logl, pair_2mom);
  gainV = update_gainV(arslds, v, logl, pair_mean, pair_2mom, ...
		       gainV);
  
  clear pair_mean;
  clear pair_2mom;
end

% convert an arhmm to an arslds

function [arslds] = arslds_from_arhmm(hmm, CovV)

S = hmm.S;
R = hmm.R;
H = R;
K = hmm.K;

prior = hmm.p';
stran = hmm.A';

pMean = zeros(H,S);
pCov  = zeros(H,H,S);

C = zeros(R,S);

A     = zeros(H,H,S);
CovH  = zeros(1,S);
CovV  = CovV*ones(1,S);
vbias = zeros(1,S);

for s=1:S
  pCov(:,:,s) = zeros(H,H);
  pCov(1,1,s) = hmm.Cov(1,1,s);
  
  CovH(s) = hmm.Cov(1,1,s);

  for r=1:R
    C(r,s) = hmm.C(1,1,r,s);
  end

  A(1,:,s)     = -C(:,s)';
  A(2:end,:,s) = eye(H-1,H);
end

arslds.S = S;
arslds.H = H;
arslds.K = K;

arslds.prior = prior;
arslds.stran = stran;
arslds.pMean = pMean;
arslds.pCov  = pCov;
arslds.A     = A;
arslds.CovH  = CovH;
arslds.CovV  = CovV;
arslds.vbias = vbias;

% update gainH

function [gain] = update_gainH(arslds, logl, pair_2mom)

S = arslds.S;
H = arslds.H;
K = arslds.K;
T = size(logl,2);
N = ceil(T / K);

A = arslds.A;

gain = zeros(S,T);

for n=1:N
  for s=1:S
    cnt  = 0;
    covh = 0;
    
    fst = max(2, K*(n-1)+1);
    lst = min(T, K*n);
    
    if logl(s,fst) ~= -inf
      for t=fst:lst
	tmp = pair_2mom(H+1,H+1,s,t) ...
	      - 2*A(1,:,s)*pair_2mom(1:H,H+1,s,t) ...
	      + A(1,:,s)*pair_2mom(1:H,1:H,s,t)*A(1,:,s)';
	
	covh = covh + tmp;
	cnt  = cnt + 1;
      end
      
      gain(s,n) = covh / cnt;
    end
  end
end

% update gainV

function [gain] = update_gainV(arslds, v, logl, pair_mean, pair_2mom, ...
			       gain)

S = arslds.S;
H = arslds.H;
K = arslds.K;
T = size(logl,2);
N = ceil(T / K);

vbias = arslds.vbias;

for n=1:N
  for s=1:S
    cnt  = 0;
    covv = 0;

    fst = max(2, K*(n-1)+1);
    lst = min(T, K*n);
    
    if logl(s,fst) ~= -inf
      for t=fst:lst
	vt  = v(:,t);
	mht = pair_mean(H+1:end,s,t);
	
	tmp = vt*vt - 2*vt*mht(1) - 2*vt*vbias(s) ...
	      + pair_2mom(H+1,H+1,s,t) + 2*mht(1)*vbias(s) ...
	      + vbias(s)*vbias(s);
	
	covv = covv + tmp;
	cnt  = cnt + 1;
      end

      gain(s,n) = covv / cnt;
    end
  end
end
