/* darray.c
 *
 * Copyright (C) 2006 Bertrand Mesot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA 02110-1301, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "darray.h"
/*-------------------------------------------------------------------------*/
darray *darray_alloc(int nd, ...) {
  int i;
  darray *a = (darray*)malloc(sizeof(darray));
  va_list args;

  a->nd  = nd;
  a->dim = (size_t*)malloc(nd * sizeof(size_t));
  a->dd  = (size_t*)malloc(nd * sizeof(size_t));
  a->own = 1;

  va_start(args, nd);

  for (i=0; i<nd; ++i)
    a->dim[i] = va_arg(args, int);

  a->dd[0] = 1;

  for (i=1; i<nd; ++i)
    a->dd[i] = a->dd[i-1] * a->dim[i-1];
    
  a->ne   = a->dd[nd-1]*a->dim[nd-1];
  a->data = (double*)calloc(a->ne, sizeof(double));

  va_end(args);

  return a;
}
/*-------------------------------------------------------------------------*/
void darray_free(darray *a) {
  if (a->own) {
    free(a->data);
    free(a->dim);
    free(a->dd);
  }

  free(a);
}
/*-------------------------------------------------------------------------*/
void darray_init(darray *a, double x) {
  int i;
  double *tmp = a->data;

  for (i=0; i<a->ne; ++i)
    *(tmp++) = x;
}
/*-------------------------------------------------------------------------*/
inline int darray_getidx(darray *a, int nd, va_list *args) {
  int i = nd, idx = 0;

  /* Here is a nice and useful application of the Duff's device */

  switch ((a->nd - nd) % 10) {
  case 0: do { idx += va_arg(*args, int) * a->dd[i++];
  case 9:      idx += va_arg(*args, int) * a->dd[i++];
  case 8:      idx += va_arg(*args, int) * a->dd[i++];
  case 7:      idx += va_arg(*args, int) * a->dd[i++];
  case 6:      idx += va_arg(*args, int) * a->dd[i++];
  case 5:      idx += va_arg(*args, int) * a->dd[i++];
  case 4:      idx += va_arg(*args, int) * a->dd[i++];
  case 3:      idx += va_arg(*args, int) * a->dd[i++];
  case 2:      idx += va_arg(*args, int) * a->dd[i++];
  case 1:      idx += va_arg(*args, int) * a->dd[i++]; } while(i < a->nd);
  }

  return idx;
}
/*-------------------------------------------------------------------------*/
void darray_set(darray *a, double x, ...) {
  int idx = 0;
  va_list args;

  va_start(args, x);
  idx = darray_getidx(a, 0, &args);
  va_end(args);

  a->data[idx] = x;
}
/*-------------------------------------------------------------------------*/
double darray_get(darray *a, ...) {
  int idx = 0;
  va_list args;

  va_start(args, a);
  idx = darray_getidx(a, 0, &args);
  va_end(args);

  return a->data[idx];
}
/*-------------------------------------------------------------------------*/
double *darray_getp(darray *a, ...) {
  int idx = 0;
  va_list args;

  va_start(args, a);
  idx = darray_getidx(a, 0, &args);
  va_end(args);

  return a->data+idx;
}
/*-------------------------------------------------------------------------*/
darray darray_subarray(darray *a, int nd, ...) {
  int idx = 0;
  va_list args;
  darray x;

  assert(nd >= 0 && nd < a->nd);

  va_start(args, nd);
  idx = darray_getidx(a, nd, &args);
  va_end(args);

  x.nd   = nd;
  x.dim  = a->dim;
  x.dd   = a->dd;
  x.own  = 0;
  x.ne   = x.dd[x.nd-1] * x.dim[x.nd-1];
  x.data = a->data + idx;

  return x;
}
/*-------------------------------------------------------------------------*/
matrix darray_getmat(darray *a, int nd, ...) {
  int idx = 0;
  va_list args;
  matrix m;
  
  assert(nd == 1 || nd == 2);

  va_start(args, nd);
  idx = darray_getidx(a, nd, &args);
  va_end(args);

  m.data = a->data+idx;
  m.own  = 0;

  m.nr = a->dim[0];
  m.nc = nd == 1 ? 1 : a->dim[1];
  m.ld = a->dd[1];

  return m;
}
