/* darray.h
 *
 * Copyright (C) 2006 Bertrand Mesot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA 02110-1301, USA.
 */

#ifndef DARRAY_H
#define DARRAY_H

#include <stdarg.h>
#include "matrix.h"

typedef struct darray_ {
  int nd;        /* number of dimension */
  size_t ne;     /* number of elments */
  size_t *dim;   /* number of elements per dimension */
  size_t *dd;    /* ... */
  short own;     /* ... */
  double *data;  /* data */
} darray;

/* Allocation / Deallocation */

darray *darray_alloc(int nd, ...);
void darray_free(darray *a);

/* Initialisation */

void darray_init(darray *a, double x);

/* Get / Set */

inline int darray_getidx(darray *a, int nd, va_list *args);
void darray_set(darray *a, double x, ...);
double darray_get(darray *a, ...);
double *darray_getp(darray *a, ...);
darray darray_subarray(darray *a, int nd, ...);
matrix darray_getmat(darray *a, int nd, ...);

#endif
