/* linalg.h
 *
 * Copyright (C) 2006 Bertrand Mesot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA 02110-1301, USA.
 */

#ifndef LINALG_H
#define LINALG_H

#include "matrix.h"

#ifdef NOCBLAS
typedef enum { NoTrans, Trans} trans;
typedef enum { Upper, Lower} uplo;
#else
#include <atlas/cblas.h>

typedef enum { NoTrans = CblasNoTrans, Trans = CblasTrans } trans;
typedef enum { Upper = CblasUpper, Lower = CblasLower } uplo;
#endif

/* Blas level 3 */

void matrix_dgemm(trans tra, trans trb, double alpha, matrix *a, 
		  matrix *b, double beta, matrix *c);

/* Blas level 2 */

void matrix_dgemv(trans tra, double alpha, matrix *a,
		  matrix *x, double beta, matrix *y);
void matrix_dtrmv(trans tra, matrix *a, matrix *x);
void matrix_dtrsv(trans tra, matrix *a, matrix *x);

/* Blas level 1 */

void matrix_daxpy(double alpha, matrix *x, matrix *y);
double matrix_ddot(matrix *x, matrix *y);

/* Addition */

void matrix_add(matrix *a, matrix *b, matrix *c);
void matrix_addm(trans tra, trans trb, matrix *a, matrix *b, matrix *c);
void matrix_addn(trans tra, trans trb, double alpha, matrix *a,
		 double beta, matrix *b, matrix *c);

/* LU stuff */

void matrix_dgetrf(matrix *a, int *p);
void matrix_dgetri(matrix *a, int *p);
void matrix_dgesv(matrix *a, matrix *b, int *p);

double matrix_lu_det(matrix *a, int *p);
double matrix_lu_lndet(matrix *a);

/* Cholesky stuff */

void matrix_dpotrf(matrix *a);

double matrix_chol_sqrtdet(matrix *a);
double matrix_chol_lnsqrtdet(matrix *a);

#endif
