/* matrix.h
 *
 * Copyright (C) 2006 Bertrand Mesot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA 02110-1301, USA.
 */

#ifndef MATRIX_H
#define MATRIX_H

#include <stdio.h>

typedef struct matrix_ {
  int nr;           /* number of rows */
  int nc;           /* number of columns */
  int ld;           /* first/last row, first/last column */
  short own;        /* does the matrix own the data */ 
  double *data;     /* data */
} matrix;

/* Allocation / Deallocation */

matrix *matrix_alloc(int nr, int nc);
matrix *matrix_calloc(int nr, int nc);
void matrix_free(matrix *m);

/* Submatrix */

matrix matrix_submatrix(matrix *m, int fr, int lr, int fc, int lc);
matrix matrix_diagonal(matrix *m);

/* Copying */

void matrix_memcpy(matrix *dest, matrix *src);
matrix *matrix_cpy(matrix *m);

/* Initialisation */

void matrix_set_identity(matrix *m);
void matrix_init(matrix *m, double x);

/* Set, get */

void matrix_set(matrix *m, int i, int j, double v);
double matrix_get(matrix *m, int i, int j);

/* Operations */

void matrix_scale(double x, matrix *m);
void matrix_transpose(matrix *m);

/* Misc */

void matrix_randint(matrix *m);
void matrix_print(int trans, matrix *m);
void matrix_fprint(FILE *file, matrix *m);

#endif
